@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2009-2015 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2009-2019 Jakub Wilk <jwilk@jwilk.net>
 #
 # This file is part of didjvu.
 #
@@ -13,6 +13,10 @@
 # FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 # for more details.
 
+'''
+didjvu core
+'''
+
 from __future__ import print_function
 
 import itertools
@@ -178,16 +182,23 @@ def generate_mask(filename, image, method, params):
     else:
         return gamera.load_image(filename)
 
-class main():
-
-    compression_info_template = (
+def format_compression_info(bytes_in, bytes_out, bits_per_pixel):
+    ratio = 1.0 * bytes_in / bytes_out
+    percent_saved = (1.0 * bytes_in - bytes_out) * 100 / bytes_in
+    msg = (
         '{bits_per_pixel:.3f} bits/pixel; '
         '{ratio:.3f}:1, {percent_saved:.2f}% saved, '
         '{bytes_in} bytes in, {bytes_out} bytes out'
+    ).format(
+        bits_per_pixel=bits_per_pixel,
+        ratio=ratio, percent_saved=percent_saved,
+        bytes_in=bytes_in, bytes_out=bytes_out,
     )
+    return msg
+
+class main(object):
 
     def __init__(self):
-        setup_logging()
         parser = cli.ArgumentParser(gamera.methods, default_method='djvu')
         parser.parse_args(actions=self)
 
@@ -199,6 +210,7 @@ class main():
                 len(o.input),
                 len(o.masks)
             )
+        setup_logging()
         ipc_logger = logging.getLogger('didjvu.ipc')
         assert logger is not None
         log_level = {
@@ -288,9 +300,7 @@ class main():
         finally:
             djvu_file.close()
         bits_per_pixel = 8.0 * bytes_out / (width * height)
-        ratio = 1.0 * bytes_in / bytes_out
-        percent_saved = (1.0 * bytes_in - bytes_out) * 100 / bytes_in
-        compression_info = self.compression_info_template.format(**locals())
+        compression_info = format_compression_info(bytes_in, bytes_out, bits_per_pixel)
         logger.info('- ' + compression_info)
         if xmp_output:
             logger.info('- saving XMP metadata')
@@ -380,9 +390,7 @@ class main():
             finally:
                 djvu_file.close()
         bits_per_pixel = float('nan')  # FIXME!
-        ratio = 1.0 * bytes_in / bytes_out
-        percent_saved = (1.0 * bytes_in - bytes_out) * 100 / bytes_in
-        compression_info = self.compression_info_template.format(**locals())
+        compression_info = format_compression_info(bytes_in, bytes_out, bits_per_pixel)
         logger.nosy(compression_info)
 
     def _bundle_complex_page(self, o, page, minidjvu_in_dir, image_filename, mask_filename, pixels):
@@ -467,9 +475,9 @@ class main():
                 finally:
                     djvu_file.close()
         bits_per_pixel = 8.0 * bytes_out / pixels
-        ratio = 1.0 * bytes_in / bytes_out
-        percent_saved = (1.0 * bytes_in - bytes_out) * 100 / bytes_in
-        compression_info = self.compression_info_template.format(**locals())
+        compression_info = format_compression_info(bytes_in, bytes_out, bits_per_pixel)
         logger.nosy(compression_info)
 
+__all__ = ['main']
+
 # vim:ts=4 sts=4 sw=4 et
