@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2010-2015 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2010-2018 Jakub Wilk <jwilk@jwilk.net>
 #
 # This file is part of didjvu.
 #
@@ -20,7 +20,7 @@ filetype detection
 class generic(object):
 
     def __new__(self, *args, **kwargs):
-        raise NotImplementedError  # <no-coverage>
+        raise NotImplementedError  # no coverage
 
     @classmethod
     def like(cls, other):
@@ -42,4 +42,10 @@ def check(filename):
                 cls = djvu_single
     return cls
 
+__all__ = [
+    'check',
+    'djvu',
+    'djvu_single',
+]
+
 # vim:ts=4 sts=4 sw=4 et
