@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2009-2015 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2009-2019 Jakub Wilk <jwilk@jwilk.net>
 #
 # This file is part of didjvu.
 #
@@ -20,17 +20,16 @@ import ctypes
 import math
 import re
 import sys
-import warnings
 
 from . import utils
 
 try:
     from PIL import Image as PIL
-except ImportError as ex:  # <no-coverage>
+except ImportError as ex:  # no coverage
     utils.enhance_import_error(ex,
-        'Python Imaging Library',
-        'python-imaging',
-        'http://www.pythonware.com/products/pil/'
+        'Pillow',
+        'python-pil',
+        'https://pypi.org/project/Pillow/'
     )
     raise
 else:
@@ -39,11 +38,11 @@ else:
 
 try:
     import gamera
-except ImportError as ex:  # <no-coverage>
+except ImportError as ex:  # no coverage
     utils.enhance_import_error(ex,
         'Gamera',
         'python-gamera',
-        'http://gamera.informatik.hsnr.de/'
+        'https://gamera.informatik.hsnr.de/'
     )
     raise
 del gamera  # quieten pyflakes
@@ -75,7 +74,7 @@ def load_image(filename):
             # https://bugs.debian.org/784374
             gamera_modes = ['1', 'I;16', 'L', 'RGB']
         elif pil_image.format == 'PNG':
-            # Gamera doesn't handle 16-bit grayscale PNG images correctly.
+            # Gamera doesn't handle 16-bit greyscale PNG images correctly.
             # https://groups.yahoo.com/neo/groups/gamera-devel/conversations/messages/2425
             gamera_modes = ['1', 'L', 'RGB']
         else:
@@ -97,7 +96,7 @@ def load_image(filename):
             # and finally removed in Pillow 3.0.0.
             # https://pillow.readthedocs.io/en/3.0.x/releasenotes/3.0.0.html#deprecated-methods
             pil_image.tostring = pil_image.tobytes
-        except AttributeError:  # <no-coverage>
+        except AttributeError:  # no coverage
             pass
         image = _from_pil(pil_image)
     image.dpi = dpi
@@ -120,7 +119,7 @@ class Argument(object):
         else:
             raise NotImplementedError(
                 'argument {0}: unsupported type {1!r}'.format(self.name, arg)
-            )  # <no-coverage>
+            )  # no coverage
         if self.type in (int, float):
             [self.min, self.max] = arg.rng
             if self.min == -gamera.args.DEFAULT_MAX_ARG_NUMBER:
@@ -137,7 +136,7 @@ class Argument(object):
             if not isinstance(self.default, self.type):
                 raise TypeError(
                     'argument {0}: type({1!r}) should be {2}'.format(self.name, self.default, self.type.__name__)
-                )  # <no-coverage>
+                )  # no coverage
         else:
             self.default = None
 
@@ -150,7 +149,7 @@ class Plugin(object):
         self.name = name
         if sys.version_info >= (2, 7):
             self.args = collections.OrderedDict()
-        else:  # <no-coverage>
+        else:  # no coverage
             # Oh well.
             self.args = {}
         for arg in plugin.args:
@@ -174,7 +173,7 @@ class Plugin(object):
             else:
                 raise NotImplementedError(
                     'method {method} does not support pixel type {pt}'.format(method=self.name, pt=image.pixel_type_name)
-                )  # <no-coverage>
+                )  # no coverage
         assert image.data.pixel_type in pixel_types
         if self._method is None:
             self._method = self._plugin()
@@ -182,7 +181,7 @@ class Plugin(object):
 
 def _load_methods():
     replace_suffix = re.compile('_threshold$').sub
-    class _methods:
+    class _methods(object):
         from gamera.plugins.threshold import abutaleb_threshold
         from gamera.plugins.threshold import bernsen_threshold
         from gamera.plugins.threshold import djvu_threshold
@@ -192,9 +191,8 @@ def _load_methods():
         # TODO: from gamera.plugins.binarization import gatos_threshold
         from gamera.plugins.binarization import niblack_threshold
         from gamera.plugins.binarization import sauvola_threshold
+        from gamera.plugins.binarization import shading_subtraction
         from gamera.plugins.binarization import white_rohrer_threshold
-        if has_version(3, 3, 1):
-            from gamera.plugins.binarization import shading_subtraction
         if has_version(3, 4, 0):
             from gamera.plugins.binarization import brink_threshold
     methods = {}
@@ -220,45 +218,21 @@ def to_pil_1bpp(image):
         image = image.to_greyscale()
     return image.to_pil()
 
-def _decref(o):  # <no-coverage>
-    '''
-    Forcibly decrease refcount of the object by 1.
-    '''
-    libc = ctypes.cdll.LoadLibrary(None)
-    memmove = libc.memmove
-    memmove.argtypes = [ctypes.py_object, ctypes.py_object, ctypes.c_size_t]
-    memmove.restype = ctypes.py_object
-    return memmove(o, None, 0)
-
-def _monkeypatch_to_raw_string():  # <no-coverage>
-    '''
-    Monkey-patch to _to_raw_string plugin function to return objects with
-    correct refcounts.
-    '''
-    from gamera.plugins import _string_io
-    old_to_raw_string = _string_io._to_raw_string
-    def fixed_to_raw_string(*args, **kwargs):
-        o = old_to_raw_string(*args, **kwargs)
-        assert sys.getrefcount(o) == 3
-        o = _decref(o)
-        assert sys.getrefcount(o) == 2
-        return o
-    _string_io._to_raw_string = fixed_to_raw_string
-    Image._to_raw_string = fixed_to_raw_string
-
 def init():
+    if not has_version(3, 3, 2):
+        raise RuntimeError('Gamera >= 3.3.2 is required')
     sys.modules['numpy'] = None
     result = _init()
     test_image = Image((0, 0), (5, 5), RGB)
     test_string = test_image._to_raw_string()
+    try:
+        sys.getrefcount
+    except AttributeError:
+        return result
     refcount = sys.getrefcount(test_string)
-    if refcount == 3:  # <no-coverage>
-        # See: https://tech.groups.yahoo.com/group/gamera-devel/message/2068
-        warnings.warn(RuntimeWarning('Working around memory leak in the Gamera library'), stacklevel=2)
-        _monkeypatch_to_raw_string()
-        fixed_test_string = test_image._to_raw_string()
-        assert sys.getrefcount(fixed_test_string) == 2
-        assert test_string == fixed_test_string
+    if refcount >= 3:  # no coverage
+        # See: https://groups.yahoo.com/neo/groups/gamera-devel/conversations/topics/2068
+        raise RuntimeError('Memory leak in Gamera')
     else:
         assert refcount == 2
     try:
